@extends('aud::password.layout')

@section('title', 'AuthStack &ndash; Reset Password')

@section('content')

    <p>Hi there,</p>
    <p>Please click in the button to reset your email.</p>

    <a href="{{ $link }}" target="_blank">
        <div class="form-group row mb-0">
            <div class="col-md-6 offset-md-4">
                <input type="submit" value="RESET">
            </div>
        </div>
    </a>

    <br>

    <p>If you cannot click in the button, copy/paste this link in your browser</p>
    <textarea cols="75" disabled>{{ $link }}</textarea>
@endsection